% Example of correcting OBS vertical channel data for tilt and compliance
% noise, using the TiSKit software package

%% Read data and metadata
net='XF';

site='B12'; pchan='EDH'; removeChannels={'HH1',pchan,'HH2'};
filename='XF.B12.2012_10_01-05_2Hz.mseed';

% Read in data (takes a lot of time at initial sample rate, so generally create
%   downsampled file with obsPy)
d=read(ts_file(filename,'MSEED'));
% Make sure that data starts at same time and is same length
d=d.synchronize();
% Plot time series
figure(1); d.plot();
    
%% Read in transfer functions
for i=1:d.nChannels,
    cmp=d.channel(i).metadata.component;
    if cmp(end)=='H', % Pressure channel
        d.channel(i).metadata.response=ts_response('SACPZ','Pa','du',sprintf('SAC_PZs_%s_%s_%s',net,site,cmp));
    else % Seismometer channel
        d.channel(i).metadata.response=ts_response('SACPZ','m','du',sprintf('SAC_PZs_%s_%s_%s',net,site,cmp));
        d.channel(i).metadata.response.inUnits='m/s^2'; % Adds appropriate poles/zeros
    end
end

%% Calculate and plot spectra and coherence
% Calculate power spectral density
wl=1024;    % Window length to use
s=d.calcSpectra('wl',wl);
figure(2); s.plot();
figure(3); s.plotcoher();

%% Find channel indices corresponding to channel names
% First the ordered list of channels to remove
for i=1:max(size(removeChannels))
    rCh=removeChannels{i};
    for j=1:d.nChannels,
        if strcmp(d.channel(j).metadata.component,rCh),
            removeOrder(i)=j;
            break;  % Channel name/address found, skip to next channel
        end
        if j==d.nChannels,
            error('Channel name "%s" not found in data',rCh);
        end
    end
end
% Then the Z channel
for j=1:d.nChannels,
    if strcmp(d.channel(j).metadata.component(3),'Z'),
        iZ=j;
        break;  % Channel name/address found, skip to next channel
    end
    if j==d.nChannels,
        error('Z channel not found');
    end
end

%% Clean data
max_freq=0.03;  % Max frequency to remove
% Clean first term
clean_xf=clean_calc(d,removeOrder(1),wl,10);
clean_xf.gooddata=clean_xf.freqs<max_freq;
s_clean1=d.calcSpectra('subtractxf',clean_xf,'wl',wl);

% Clean first and second terms
clean_xf=clean_calc(d,removeOrder(1:2),wl,10);
clean_xf.gooddata=clean_xf.freqs<max_freq;
s_clean2=d.calcSpectra('subtractxf',clean_xf,'wl',wl);

% Clean all three terms
clean_xf=clean_calc(d,removeOrder,wl,10);
clean_xf.gooddata=clean_xf.freqs<max_freq;
s_clean3=d.calcSpectra('subtractxf',clean_xf,'wl',wl);


%% Plot spectra and coherences from each step
close all
figure(1); clf; d.plot();
figure(2); clf; s.plot();
figure(3); clf; s.plotcoher();
figure(4); clf; s_clean1.plotcoher();
figure(5); clf; s_clean2.plotcoher();
figure(6); clf; s_clean3.plotcoher();
figure(7); clf; 
    semilogx(s.freqs,10*log10(s.spectdata(iZ,:)),'r');
    hold on
    semilogx(s_clean1.freqs,10*log10(s_clean1.spectdata(iZ,:)),'g');
    semilogx(s_clean2.freqs,10*log10(s_clean2.spectdata(iZ,:)),'m');
    semilogx(s_clean3.freqs,10*log10(s_clean3.spectdata(iZ,:)),'b');
    xlabel('Frequency (Hz)');
    ylabel('Acceleration PSD (dB ref (1 m/s^2)^2/Hz)')
    title(sprintf('Network %s, Site %s',net,site));
    legend({'Z',sprintf('Z-%s',removeChannels{1}), ...
                       sprintf('Z-%s-%s',removeChannels{1},removeChannels{2}), ...
                       sprintf('Z-%s-%s-%s',removeChannels{1},removeChannels{2},removeChannels{3})}, ...
                       'Location','northwest')
    axis([.001 0.5 -180 -80])
    hold off
